@@ -21,12 +21,6 @@
  */
 
 #include <config.h>
-#include <libgnome/libgnome.h>
-#include <libgnomevfs/gnome-vfs-types.h>
-#include <libgnomevfs/gnome-vfs-file-info.h>
-#include <libgnomevfs/gnome-vfs-ops.h>
-#include <libgnomevfs/gnome-vfs-uri.h>
-
 #include <stdio.h>
 #include <sys/stat.h>   
 #include <sys/types.h>
@@ -35,9 +29,18 @@
 #include <dirent.h>
 #include <string.h>
 
+#include <libgnome/libgnome.h>
+#include <libgnomevfs/gnome-vfs-types.h>
+#include <libgnomevfs/gnome-vfs-file-info.h>
+#include <libgnomevfs/gnome-vfs-ops.h>
+#include <libgnomevfs/gnome-vfs-uri.h>
+#include <libgnomevfs/gnome-vfs-utils.h>
+#include <gtk/gtk.h>
+
 #include "typedefs.h"
 #include "thumb-cache.h"
 #include "file-utils.h"
+#include "gtk-utils.h"
 
 
 static gboolean
@@ -76,49 +79,49 @@ get_aa_name (gchar *filename)
 
 
 static gchar*
-get_escaped_dir_name (gchar *dir_path)
+get_escaped_dir_name (const char *dir_path)
 {
-	const gchar *dir_name;
-	gchar *dir_container;
-	gchar *e_name;
-	gchar *e_path;
-	gchar *result;
-	
-	if (dir_path == NULL) 
+	const char *dir_name;
+	char *dir_container;
+	char *e_name;
+	char *e_path;
+	char *result;
+
+	if (dir_path == NULL)
 		return NULL;
 
 	dir_name = file_name_from_path (dir_path);
 	dir_container = remove_level_from_path (dir_path);
-
+	
 	if (strcmp (dir_container, "/") != 0) {
 		char *tmp;
 		char *e_name_tmp;
-
+		
 		tmp = g_strconcat (dir_container, "/", NULL);
 		g_free (dir_container);
 		dir_container = tmp;
-
-		e_name_tmp = gnome_vfs_escape_string  (dir_name);
-		e_name     = gnome_vfs_escape_slashes (e_name_tmp);
+		
+		e_name_tmp = gnome_vfs_escape_string (dir_name);
+		e_name = gnome_vfs_escape_slashes (e_name_tmp);
 		g_free (e_name_tmp);
-
-		e_path     = gnome_vfs_escape_slashes (dir_container);
+		
+		e_path = gnome_vfs_escape_slashes (dir_container);
 	} else {
 		e_name = NULL;
 		e_path = NULL;
 	}
 	g_free (dir_container);
-
+	
 	result = g_strconcat (g_get_home_dir (),
 			      "/.nautilus/thumbnails/",
 			      "file:%2F%2F",
 			      e_path,
 			      e_name,
 			      NULL);
-
+	
 	g_free (e_name);
 	g_free (e_path);
-
+	
 	return result;
 }
 
@@ -180,33 +183,6 @@ cache_get_nautilus_thumbnail_file (const gchar *source)
 }
 
 
-gchar *
-cache_get_gthumb_cache_name (const gchar *source) 
-{
-	gchar *path;
-	gchar *directory;
-	const gchar *filename;
-
-	if (!source) return NULL;
-
-	directory = remove_level_from_path (source);
-	filename = file_name_from_path (source);
-
-	path = g_strconcat (g_get_home_dir(), 
-			    "/", 
-			    RC_THUMBS_DIR, 
-			    directory,
-			    "/",
-			    filename, 
-			    CACHE_THUMB_EXT, 
-			    NULL);
-
-	g_free (directory);
-
-	return path;
-}
-
-
 gchar *
 cache_get_nautilus_cache_name (const gchar *source) 
 {
@@ -233,25 +209,29 @@ cache_get_nautilus_cache_name (const gchar *source)
 }
 
 
+gchar *
+cache_get_nautilus_cache_dir (const gchar *source) 
+{
+	return get_escaped_dir_name (source);
+}
+
+
 void
 cache_copy (const gchar *src,
 	    const gchar *dest)
 {
-	gchar *cache_src;
-	gchar *cache_dest;
-
-	cache_src = cache_get_gthumb_cache_name (src);
-	if (path_is_file (cache_src)) {
-		cache_dest = cache_get_gthumb_cache_name (dest);
-		file_copy (cache_src, cache_dest);
-		g_free (cache_dest);
-	}
-	g_free (cache_src);
+	char   *cache_src;
+	time_t  dest_mtime = get_file_mtime (dest);
 
 	cache_src = cache_get_nautilus_cache_name (src);
 	if (path_is_file (cache_src)) {
-		cache_dest = cache_get_nautilus_cache_name (dest);
-		file_copy (cache_src, cache_dest);
+		char *cache_dest = cache_get_nautilus_cache_name (dest);
+
+		if (path_is_file (cache_dest)) 
+			unlink (cache_dest);
+		if (file_copy (cache_src, cache_dest))
+			set_file_mtime (cache_dest, dest_mtime);
+
 		g_free (cache_dest);
 	}
 	g_free (cache_src);
@@ -259,24 +239,22 @@ cache_copy (const gchar *src,
 
 
 void
-cache_move (const gchar *src,
-	    const gchar *dest)
+cache_move (const char *src,
+	    const char *dest)
 {
-	gchar *cache_src;
-	gchar *cache_dest;
-
-	cache_src = cache_get_gthumb_cache_name (src);
-	if (path_is_file (cache_src)) {
-		cache_dest = cache_get_gthumb_cache_name (dest);
-		file_move (cache_src, cache_dest);
-		g_free (cache_dest);
-	}
-	g_free (cache_src);
+	char   *cache_src;
+	time_t  dest_mtime = get_file_mtime (dest);
 
 	cache_src = cache_get_nautilus_cache_name (src);
+
 	if (path_is_file (cache_src)) {
-		cache_dest = cache_get_nautilus_cache_name (dest);
-		file_move (cache_src, cache_dest);
+		char *cache_dest = cache_get_nautilus_cache_name (dest);
+
+		if (path_is_file (cache_dest)) 
+			unlink (cache_dest);
+		if (file_move (cache_src, cache_dest))
+			set_file_mtime (cache_dest, dest_mtime);
+		
 		g_free (cache_dest);
 	}
 	g_free (cache_src);
@@ -286,11 +264,7 @@ cache_move (const gchar *src,
 void
 cache_delete (const gchar *filename)
 {
-	gchar *cache_name;
-
-	cache_name = cache_get_gthumb_cache_name (filename);
-	unlink (cache_name);
-	g_free (cache_name);
+	char *cache_name;
 
 	cache_name = cache_get_nautilus_cache_name (filename);
 	unlink (cache_name);
@@ -298,96 +272,17 @@ cache_delete (const gchar *filename)
 }
 
 
-void
-cache_remove_old_previews (const gchar *dir,
-			   gboolean recursive,
-			   gboolean clear_all)
-{
-	visit_rc_directory (RC_THUMBS_DIR,
-			    CACHE_THUMB_EXT,
-			    dir,
-			    recursive,
-			    clear_all);
-}
-
-
-/* ----- cache_remove_old_previews_async implememtation. ------ */
-
-
-typedef struct {
-	gboolean recursive;
-	gboolean clear_all;
-} CacheRemoveData;
-
-
-static void nautilus_cache_remove_old_previews_async (gboolean recursive,
-						      gboolean clear_all);
-
-
-static void
-cache_remove_done (const GList *dir_list,
-		   gpointer data)
-{
-	CacheRemoveData *crd = data;
-
-	if (crd->clear_all) {
-		const GList *scan;
-		for (scan = dir_list; scan; scan = scan->next) {
-			gchar *dir = scan->data;
-			rmdir (dir);
-		}
-	}
-
-	nautilus_cache_remove_old_previews_async (crd->recursive,
-						  crd->clear_all);
-	g_free (crd);
-}
-
-
-static void 
-check_cache_file (gchar *real_file, 
-		  gchar *rc_file, 
-		  gpointer data)
-{
-	CacheRemoveData *crd = data;
-
-	if (crd->clear_all || ! path_is_file (real_file)) 
-		if ((unlink (rc_file) < 0))
-			g_warning ("Cannot delete %s\n", rc_file);
-}
-
-
-void
-cache_remove_old_previews_async (const gchar *dir,
-				 gboolean recursive,
-				 gboolean clear_all)
-{
-	CacheRemoveData *crd;
-
-	crd = g_new (CacheRemoveData, 1);
-	crd->recursive = recursive;
-	crd->clear_all = clear_all;
-
-	visit_rc_directory_async (RC_THUMBS_DIR,
-				  CACHE_THUMB_EXT,
-				  dir,
-				  recursive,
-				  check_cache_file,
-				  cache_remove_done,
-				  crd);
-}
-
-
 /* ----- nautilus_cache_remove_old_previews_async implememtation. ------ */
 
 
 typedef struct {
-	gboolean recursive;
-	gboolean clear_all;
-	GList *dirs;
-	GList *visited_dirs;
-	gchar *nautilus_thumb_dir;
-	int nautilus_thumb_dir_l;
+	gboolean   recursive;
+	gboolean   clear_all;
+	GList     *dirs;
+	GList     *visited_dirs;
+	gchar     *nautilus_thumb_dir;
+	int        nautilus_thumb_dir_l;
+	GtkWidget *dialog;
 } NautilusCacheRemoveData;
 
 
@@ -410,6 +305,8 @@ nautilus_cache_data_free (NautilusCacheRemoveData *ncrd)
 	if (ncrd->nautilus_thumb_dir)
 		g_free (ncrd->nautilus_thumb_dir);
 
+	gtk_widget_destroy (ncrd->dialog);
+
 	g_free (ncrd);
 }
 
@@ -426,50 +323,73 @@ static void visit_dir_async (const gchar *dir,
 
 static gchar *
 get_real_name_from_nautilus_cache (NautilusCacheRemoveData *ncrd,
-				   char *cache_name_full_path)
+				   const char *cache_name_full_path)
 {
-	int cache_name_l;
-	int real_name_l;
-	int scan_l;
-	int n;
-	char *cache_name;
+	const char *cache_path;
+	const char *cache_name;
+	char *cache_dir;
+	char *e_cache_dir;
+	int   cache_path_l, real_name_l;
 	char *real_name;
-	char *scan;
+	char *tmp;
 
 	if (strlen (cache_name_full_path) < ncrd->nautilus_thumb_dir_l + 1)
 		return NULL;
 
-	cache_name = cache_name_full_path + ncrd->nautilus_thumb_dir_l + 1;
+	cache_path = cache_name_full_path + ncrd->nautilus_thumb_dir_l + 1;
+	cache_path_l = strlen (cache_path);
 
-	cache_name_l = strlen (cache_name);
-	if (cache_name_l < ESC_FILE_PREFIX_L)
+	if (cache_path_l < ESC_FILE_PREFIX_L)
 		return NULL;
 
-	if (strncmp (cache_name, ESC_FILE_PREFIX, ESC_FILE_PREFIX_L) != 0)
+	if (strncmp (cache_path, ESC_FILE_PREFIX, ESC_FILE_PREFIX_L) != 0)
 		return NULL;
 
-	real_name = g_new (char, strlen (cache_name) + 1);
-	n = 0;
+	cache_path = cache_path + ESC_FILE_PREFIX_L;
+	cache_dir = remove_level_from_path (cache_path);
+	cache_name = file_name_from_path (cache_path);
 
-	scan_l = cache_name_l - ESC_FILE_PREFIX_L;
-	for (scan = cache_name + ESC_FILE_PREFIX_L; *scan != 0; ) 
-		if ((scan_l >= ESC_SLASH_L) 
-		    && strncmp (scan, ESC_SLASH, ESC_SLASH_L) == 0) {
-			real_name[n++] = '/';
-			scan   += ESC_SLASH_L;
-			scan_l -= ESC_SLASH_L;
-		} else {
-			real_name[n++] = *scan;
-			scan++;
-			scan_l--;
+	tmp = gnome_vfs_unescape_string (cache_dir, NULL);
+	e_cache_dir = gnome_vfs_unescape_string (tmp, NULL);
+	g_free (tmp);
+	g_free (cache_dir);
+
+	real_name = g_strconcat (e_cache_dir, "/", cache_name, NULL);
+	g_free (e_cache_dir);
+
+#if 0
+	/* --- 8< --- */
+
+	{
+		char *test_file;
+		char *test_dir;
+		char *test_e_dir;	
+		char *test;
+
+		test_file = g_strdup (file_name_from_path (real_name));
+		test_dir = remove_level_from_path (real_name);
+		test_e_dir = get_escaped_dir_name (test_dir);
+		test = g_strconcat (test_e_dir, "/", test_file, NULL);
+		
+		if (strcmp (test, cache_name_full_path) != 0) {
+			g_error ("%s --> %s --> %s\n", cache_name_full_path, real_name, test);
 		}
-	real_name[n] = 0;
+
+		g_free (test_file);
+		g_free (test_dir);
+		g_free (test_e_dir);
+		g_free (test);
+	}
+
+	/* --- 8< --- */
+#endif
+
+	/* Remove the png extension. */
 
 	real_name_l = strlen (real_name);
-	real_name[real_name_l - 4] = 0; /* 4 = strlen (".png") */
-
-	if (! file_is_image (real_name, TRUE))
-		real_name[real_name_l - 4] = '.';
+        real_name[real_name_l - 4] = 0; /* 4 = strlen (".png") */
+        if (! file_is_image (real_name, TRUE))
+                real_name[real_name_l - 4] = '.';
 
 	return real_name;
 }
@@ -497,15 +417,16 @@ path_list_done_cb (PathListData *pld,
 	}
 
 	for (scan = pld->files; scan; scan = scan->next) {
-		rc_file = (gchar*) scan->data;
+		rc_file = (char*) scan->data;
 		real_file = get_real_name_from_nautilus_cache (ncrd, rc_file);
 
 		if (real_file == NULL)
 			continue;
 
-		if (ncrd->clear_all || ! path_is_file (real_file)) 
+		if (ncrd->clear_all || ! path_is_file (real_file)) {
 			if ((unlink (rc_file) < 0)) 
 				g_warning ("Cannot delete %s\n", rc_file);
+		}
 
 		g_free (real_file);
 	}
@@ -533,7 +454,7 @@ path_list_done_cb (PathListData *pld,
 		return;
 	}
 
-	sub_dir = (gchar*) ncrd->dirs->data;
+	sub_dir = (char*) ncrd->dirs->data;
 	ncrd->dirs = g_list_remove_link (ncrd->dirs, ncrd->dirs);
 
 	ncrd->visited_dirs = g_list_prepend (ncrd->visited_dirs,
@@ -560,6 +481,12 @@ nautilus_cache_remove_old_previews_async (gboolean recursive,
 					  gboolean clear_all)
 {
 	NautilusCacheRemoveData *ncrd;
+	const char              *message;
+
+	if (clear_all)
+		message = _("Deleting all thumbnails, wait please...");
+	else
+		message = _("Deleting old thumbnails, wait please...");
 
 	ncrd = g_new (NautilusCacheRemoveData, 1);
 
@@ -573,5 +500,27 @@ nautilus_cache_remove_old_previews_async (gboolean recursive,
 						NULL);
 	ncrd->nautilus_thumb_dir_l = strlen (ncrd->nautilus_thumb_dir);
 
+	ncrd->dialog =  _gtk_message_dialog_new (NULL,
+						 GTK_DIALOG_MODAL,
+						 GTK_MESSAGE_INFO,
+						 message,
+						 GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
+						 NULL);
+	g_signal_connect_swapped (G_OBJECT (ncrd->dialog),
+				  "response",
+				  G_CALLBACK (gtk_widget_hide),
+				  ncrd->dialog);
+
+	gtk_widget_show (ncrd->dialog);
+
 	visit_dir_async (ncrd->nautilus_thumb_dir, ncrd);
 }
+
+
+void
+cache_remove_old_previews_async (const char *dir,
+				 gboolean    recursive,
+				 gboolean    clear_all)
+{
+	nautilus_cache_remove_old_previews_async (recursive, clear_all);
+}
